<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

include('php/mimetypes.inc');

class MurderMime
{
	static private $instance = false;
	private $overrides = array();
	
	private function __construct()
	{
	}

	static public function getInstance()
	{
		if(!MurderMime::$instance)
		{
			MurderMime::$instance = new MurderMime();
		}
		return MurderMime::$instance;
	}

	static public function override($type, $ending, $with)
	{
		MurderMime::getInstance()->_override($type, $ending, $with);
	}

	protected function _override($type, $ending, $with=false)
	{
		if(!isset($this->overrides[$type]))
		{
			if(!$with) return false;
			$this->overrides[$type] = array();
		}


		if($with) 
			$this->overrides[$type][$ending] = $with;
		else 
			return safeGet($this->overrides[$type],$ending);
	}

	public function file_type($file)
	{
		$type =  mime_content_type($file);
		$ending = $this->_extension($file);
		if(!$ending) return $type;
		$override = $this->_override($type, $ending);
		if($override) return $override;
		return $type;
	}

	private function _extension($file)
	{
		$n = strrpos($file, '.');
		if($n > 0) return substr($file, $n+1);
		return false;
	}

	static public function resolve($available, $accepted=false)
	{
		if(!$accepted)
		{
			return $available[0];	// todo, local q
		}
		$types = array();
		$types_t = explode(',', $accepted);
		foreach ($types_t as $t)
		{
			$t_array = explode(';', $t);
			if(count($t_array) < 2) $t_array[1]=1;
			else $t_array[1] = substr($t_array[1], strpos($t_array[1], '=')+1);	// XXX is this always right?
			$types[trim($t_array[0])] = $t_array[1];
		}
		$best = false;
		$bscore= 0;
		$default = safeGet($available, '*');
		foreach($available as $option)
		{
			$this_score = false;
			if(isset($types[$option]))
			{
				$this_score = $types[$option];
			}
			else
			{
				$major = substr($option, 0, strpos($option, '/')) . '/*';
				if(isset($types[$major]))
				{
					$this_score = $types[$major];
				}
				else
				{
					$this_score = safeGet($types, "*/*", false);
				}
			}
			if($this_score === false) continue;
			if($this_score >= $bscore)
			{
				if($best !== $default)
				{
					$bscore = $this_score;
					$best = trim($option);
				}
			}
		} 
		if($bscore > 0) return $best;
		return false;
	}

	static public function isAccepted($type, $accepted=false)
	{
		if(!$accepted) return true;
		$a = MurderMime::resolve(array($type), $accepted);
		if($a == $type) return true;
		return false;
	}
}

function murder_mime_content_type($file)
{
	return MurderMime::getInstance()->file_type($file);
}



?>
