<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

class PluginStore
{
	static private $instance = false;
	private $bIsInitialized = false;
	private $subhandlers = array();	# URL handlers
	private $menu = array();	# Menu Links

	private function __construct() { }

	private function init()
	{
		if($this->bIsInitialized) return;
		$this->bIsInitialized = true;
		$this->_addPlugins(config('PLUGIN_DIR').'/');
	}

	private function _addPlugins($directory)
	{
		$plugins = scandir($directory);
		if($plugins == false) return;
		foreach($plugins as $p)
		{
			if(substr($p, 0, 1) == '.') continue;
			$path = $directory.$p;
			if(is_dir($path))
			{
				include_once($path.'/murder.inc');
			}
		}
	}

	static public function getInstance()
	{
		if(!PluginStore::$instance)
		{
			PluginStore::$instance = new PluginStore();
			PluginStore::$instance->init();
		}
		return PluginStore::$instance;
	}

	# $arg1 == url and $arg2 == text
	# OR
	# $arg1 == callback
	public function addMenu($match, $arg1, $arg2=false)
	{
		$match = '/' . str_replace('/','\/',$match) . '/';
		if(!isset($this->menu[$match]))
		{
			$this->menu[$match] = array(); 
		}
		$item = $arg2?array('url'=>$arg1, 'text'=>$arg2):array('callback' => $arg1);
		$this->menu[$match][] = $item;
	}

	public function getMenuItems($uri)
	{
		$items = array();
		foreach(array_keys($this->menu) as $key)
		{
			if(0==preg_match($key, $uri)) continue;
			$mi = $this->menu[$key];
			$items = array_merge($items, $mi);
		}
		return $items;
	}

	public function addHandlers($handlers, $sub)
	{
		if(!$handlers || !is_array($handlers)) return;
		while(list($type, $h) = each($handlers))
		{
			$this->setURLHandler($sub, $h, $type);
		}
	}

	public function getURLHandler($sub, $acceptedTypes='*/*')
	{
		$subHandlers = safeGet($this->subhandlers, $sub);
		if(!$subHandlers || !is_array($subHandlers)) return false;
		$type = MurderMime::resolve(array_keys($subHandlers), $acceptedTypes);
		if($type)
		{
			$handler = safeGet($subHandlers,$type);
			return array('handler' => $handler, 'type' => $type);
		}
		return false;
	}

	public function setURLHandler($sub, $handler, $type)
	{
		if(!isset($this->subhandlers[$sub]))
		{
			$this->subhandlers[$sub] = array(); 
		}
		$this->subhandlers[$sub][$type] = $handler;
	}
}
