<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

class PageRenderer
{
	public function __construct($page, $args)
	{
		
	}

	public function processRequest($page) { }

	protected function checkModification($page)
	{
		$headers = getallheaders();
		if(!$headers || !is_array($headers)) return;
		foreach($headers as $name => $value)
		{
			if(!strcasecmp($name, 'if-modified-since'))
			{
				$old = strtotime($value);
				$new = $page->getLastModified();
				if($new <= $old)
				{
					// Unmodified! drop and roll!
					$page->setStatus(304);
				}
				return;
			}
		}
	}

	public function sendHeaders($page)
	{
		$modified = $page->getLastModified(true);
		// We can try and do something smart with last modified
		if($modified) $this->checkModification($page);
		if(!MurderMime::isAccepted($page->getContentType(), $page->acceptableContentTypes()))
		{
			$page->setStatus(406);
		}

		header('HTTP/1.1 ' . $page->getStatusCode() . ' ' . $page->getStatusMessage());
		header('Content-type: ' . $page->getContentType() );
		if($page->getMaxAge())
		{
			header('Cache-Control: max-age=' . $page->getMaxAge());
		}
		if($page->getVary())
		{
			header('Vary: ' . $page->getVary());
		}
		if($modified)
		{
			header('Last-Modified: ' . $modified);
		}
	}

	protected function renderGoBack($page)
	{
		$user = $page->getUser();
		if($user)
		{
			$login = $user->getLogin();
			$url   = "/~${login}/";
			if($_SERVER['REQUEST_URI'] != $url)
			{
				print "<a href=\"${url}\">${login}</a>";
				return;
			}
		}
		print '<a href="/">Home</a>';
	}

	public function startPage($page)
	{
?>
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" >
<head><title><?php echo htmlspecialchars($page->getTitle()); ?></title>
<?php foreach($page->getStylesheets() as $style) { ?>
	<link type="text/css" rel="stylesheet" href="<?php print htmlspecialchars($style); ?>" />
<?php	} ?>
</head>
<body>
<div id="canvas">
<div id="heading">
	<?php if($_SERVER['REQUEST_URI'] != '/') { ?>
<div id="goho"><?php $this->renderGoBack($page); ?></div>
	<?php } ?>
	<h1><?php print $page->getTitle(); ?></h1>
</div>
<div id="page">
<?php
		$msgs = $page->msg();
		if($msgs && is_array($msgs) && sizeof($msgs))
		{
			print "<div id='MMSGS'>";
			foreach($msgs as $m)
			{
				print "<div class='mmsg'>${m}</div>";
			}
			print "</div>";
		}
	}

	public function endPage($page)
	{
?><div id="menu"><ul><?php
	foreach(PluginStore::getInstance()->getMenuItems($_SERVER['REQUEST_URI']) as $item)
	{
		$callback = safeGet($item, 'callback');
		$out = '';
		if($callback)
		{
			$out = $callback();
		} 
		else
		{
			$url = $item['url']; $text = $item['text'];
			$link = "<a href='${url}'>${text}</a>";
			$out = ($url == $_SERVER['REQUEST_URI'] ? $text : $link );
		}
		if($out) print "<li>${out}</li>";
	}
?></ul></div>
</div></div><div id="backdrop" /></body></html>
<?php
	}

	public function mainContent($page)
	{
		print "<p>" . $page->getStatusMessage() ."</p>";
	}
}

?>
