<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>
require_once('php/murder/plugin.inc'); 

class DefaultRenderer extends PageRenderer
{

	public function processRequest($page)
	{
		if(($user=$page->getUser()) && !$user->isHidden())
		{
			$page->setTitle( $user->getName() );
			$user->getKnownPeople();	# load last known for their last modification
			$page->setLastModified($user->getLastModified());
		}
	}

	public function mainContent($page)
	{

		if(($user=$page->getUser()) && !$user->isHidden())
		{
			$this->showUser($user);
			$this->showKnows($user);
			print "<div class='clear' />";
		}
		else
		{
			echo "<ul>";
			foreach(UserStore::getInstance()->getUsers() as $u)
			{
				$login = $u->getLogin();
				echo "<li><a href='/~${login}/'>" . $u->getName() ."</a></li>\n";
			}
			echo "</ul>";
		}

	}

	static private function _sortKnowns($a, $b)
	{
		$aHasImage = $a->getThumbnailURI();
		$bHasImage = $b->getThumbnailURI();
		if($aHasImage && ! $bHasImage) return -1;
		if($bHasImage && ! $aHasImage) return 1;
		return($a->getLogin() < $b->getLogin() ? -1 : 1);
	}

	protected function showKnows($user)
	{
		$k=$user->getKnownPeople();
		if(!$k) return;
		uasort($k, 'DefaultRenderer::_sortKnowns');
		echo "<div id='knows'>";
		echo "<h3>Knows</h3>";
		foreach($k as $p)
		{
			$this->_showKnownPerson($p);
		}
		echo "</div>";
	}

	private function _showKnownPerson($person)
	{
		print "<div class='known'>";
		if($person->getThumbnailURI())
		{
			$uri= htmlspecialchars($person->getThumbnailURI());
			$alt= htmlspecialchars($person->getNickname());
			print "<img src='${uri}' alt='${alt}' />";
		}
		print "<p>" . $person->getNickname() . "</p>";
		print "</div>";
	}

	private function _showRow($key, $value, $href=false)
	{
		if(!$value || !trim($value)) return;
		$ekey = htmlspecialchars($key);
		$evalue = htmlspecialchars($value);
		if($href)
		{
			$evalue = "<a href=\"${href}\">${evalue}</a>";
		}
		print "<tr><th>${ekey}:</th><td>${evalue}</td></tr>";
	}

	protected function showUser($user)
	{
		echo "<div id='profile'>";
		echo "<div id='profileData'>";
		echo "<h2>" . htmlspecialchars($user->getName()) . " ( " . $user->getLogin() . " )</h2>";
		echo "<table>";
		$this->_showrow('Location' , $user->getRoom());
		$this->_showrow('Home' , $user->getHomePhone(), 'tel:'.$user->getHomePhone());
		$this->_showrow('Work' , $user->getWorkPhone(), 'tel:'.$user->getWorkPhone());
		$this->_showrow('Other' , $user->getOther(), 'tel:'.$user->getOther());
		$this->_showrow('SIP' , $user->getSIP(), 'sip:' . $user->getSIP());
		$this->_showrow('eMail' , $user->getEMail(), 'mailto:' . $user->getEMAIL());
		$this->_showrow('Jabber' , $user->getJabberID(), 'xmpp:' . $user->getJabberID());
		echo "</table>";
		echo "<pre>plan: " . $user->getPlan() . '</pre>';
		echo "<pre>project: " . $user->getProject() . '</pre>';
		if($user->getPGPKey()) echo "<p><a href='pgp.key'>PGP key</a></p>";
		if($user->hasWeb()) echo "<p><a href='". htmlspecialchars($user->getHomepage()) ."'>Web site</a></p>";
		echo "</div>"; // profileData
		if($user->hasImage()) echo "<div id='userImage'><img src='" . $user->getDepictionURI() . "' alt='image' /></div>";
		echo "</div>"; // profile
	}
}
registerp(array('text/xml' => "DefaultRenderer"), "/");
registerp(array('application/xhtml+xml' => "DefaultRenderer"), "/");
registerp(array('*' => "DefaultRenderer"), "/"); # make this the default in case of a priority conflict

?>
