<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

class FoafRDFRenderer extends DefaultRenderer
{
	private $user;
	private $bIsGood;
	private $contentType;

	public function __construct($page, $arguments, $contentType)
	{
		parent::__construct($page, $arguments);
		$this->user = $page->getUser();
		$this->contentType = $contentType;
		if(!$this->user || $this->user->isHidden()) 
		{
			$this->bIsGood = false;
		}
		else
		{
			$this->bIsGood = true; 
		}
	}

	public function mainContent($page)
	{
		if(!$this->bIsGood)
		{
			parent::mainContent($page);
			return;
		}
		$xml = FoafRDFRenderer::_buildContent($this->user, 'me', false);
		FoafRDFRenderer::_dumpXML($xml);
	}

	// TODO: Clean this up a bit more.  Currently it needs a very wide terminal to edit.
	static private function _buildContent($user, $id, $hideSensitive)
	{
		$r = FoafRDFRenderer::_buildNode('foaf:Person');
		FoafRDFRenderer::_setAttribute($r, 'rdf:ID', $id);
		if($user->getName()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:name', $user->getName()));
		if($user->getNickname()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:nick', $user->getNickname()));
		if($user->getEMail())
		{
			$mail = 'mailto:' .$user->getEMail();
			if(!$hideSensitive) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:mbox', $mail));
			FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:mbox_sha1sum', sha1($mail)));
		}
		if(!$hideSensitive && $user->getJabberID()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:jabberID', 'xmpp:'.$user->getJabberID()));
		if(!$hideSensitive && $user->getWorkPhone()) 
		{
			$n = FoafRDFRenderer::_buildNode('foaf:phone');
			FoafRDFRenderer::_setAttribute($n, 'rdf:resource', 'tel:'.$user->getWorkPhone());
			FoafRDFRenderer::_addChild($n, FoafRDFRenderer::_buildNode('dc:description', 'work'));
			FoafRDFRenderer::_addChild($r, $n);
		}
		if(!$hideSensitive && $user->getHomePhone()) 
		{
			$n = FoafRDFRenderer::_buildNode('foaf:phone');
			FoafRDFRenderer::_setAttribute($n, 'rdf:resource', 'tel:'.$user->getHomePhone());
			FoafRDFRenderer::_addChild($n, FoafRDFRenderer::_buildNode('dc:description', 'home'));
			FoafRDFRenderer::_addChild($r, $n);
		}
		if(!$hideSensitive && $user->getSIP()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:phone', 'sip:'.$user->getSIP()));
		if($user->getOnelineBio()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('bio:olb', $user->getOnelineBio()));
		if($user->getPlan()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:plan', $user->getPlan()));
		if($user->getNameGiven()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:givenName', $user->getNameGiven()));
		if($user->getNameFamily()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:familyName', $user->getNameFamily()));
		if($user->getHonorificPrefix()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:honorificPrefix', $user->getHonorificPrefix()));
		if($user->getOpenID()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:openid', $user->getOpenID()));
		if(!$hideSensitive && $user->getGender()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:gender', $user->getGender()));
		if(!$hideSensitive && $user->getBirthdayAnniversary()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:birthday', $user->getBirthdayAnniversary()));
		if(!$hideSensitive && $user->getBirthday())
		{
			FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:dateOfBirth', $user->getBirthday()));
			$n = FoafRDFRenderer::_buildNode('bio:event');
			$m = FoafRDFRenderer::_buildNode('bio:Birth');
			$q = FoafRDFRenderer::_buildNode('bio:date', $user->getBirthday());
			FoafRDFRenderer::_addChild($m, $q);
			FoafRDFRenderer::_addChild($n, $m);
			FoafRDFRenderer::_addChild($r, $n);
		}
		if($user->getHomepage()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:homepage', $user->getHomepage()));
		if($user->getWeblog()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:weblog', $user->getWeblog()));
		if($user->getDepictionURI()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:depiction', $user->getDepictionURI()));
		if($user->getThumbnailURI()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:thumbnail', $user->getThumbnailURI()));
		if($user->getMyersBriggs()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:myersBrigs', $user->getMyersBriggs()));
		if($user->getGeekCode()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:geekCode', $user->getGeekCode()));
		if(!$hideSensitive && $user->getChatAIM()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:aimChatID', $user->getChatAIM()));
		if(!$hideSensitive && $user->getChatMSN()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:msnChatID', $user->getChatMSN()));
		if(!$hideSensitive && $user->getChatICQ()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:icqChatID', $user->getChatICQ()));
		if(!$hideSensitive && $user->getChatYahoo()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:yahooChatID', $user->getChatYahoo()));
		if(!$hideSensitive && $user->getSkype()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildNode('foaf:skypeID', $user->getSkype()));
		if($user->getWishlist()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('foaf:tipjar', $user->getWishlist()));
		if($user->getOtherFOAF()) FoafRDFRenderer::_addChild($r, FoafRDFRenderer::_buildResourceNode('rdfs:seeAlso', $user->getOtherFOAF()));

		$knows = $user->getKnownPeople();
		if($knows && is_array($knows))
		{
			$n = FoafRDFRenderer::_buildNode('foaf:knows');
			foreach($knows as $name => $person)
			{
				$k = FoafRDFRenderer::_buildContent($person, $name, true);
				FoafRDFRenderer::_addChild($n, $k);
			}
			FoafRDFRenderer::_addChild($r, $n);
		}
		return $r;
	}

	// short cut for creating a node and adding an rdf:resource
	static private function _buildResourceNode($name, $resource)
	{
		$node = FoafRDFRenderer::_buildNode($name);
		FoafRDFRenderer::_setAttribute($node, 'rdf:resource', $resource);
		return $node;
	}

	static private function _buildNode($name, $value = '')
	{
		return array( 'name' => $name, 'value' => $value);
	}

	static private function _setAttribute(&$node, $name, $value)
	{
		if(!isset($node['attrs'])) $node['attrs'] = array();
		$node['attrs'][$name] = $value;
	}

	static private function _addChild(&$parent, &$child)
	{
		if(!isset($parent['children']))
			$parent['children'] = array();
		$parent['children'][] = $child;
	}

	static private function _dumpXML($xml)
	{
		if(!$xml || !is_array($xml)) return;
		$n = htmlspecialchars(safeGet($xml, 'name'));
		$a = safeGet($xml, 'attrs');
		$v = htmlspecialchars(safeGet($xml, 'value'));
		$c = safeGet($xml, 'children');
		print "<${n}";
		if($a && is_array($a)) foreach($a as $ak => $av) {
			print ' ' . htmlspecialchars($ak) . '=' . '"' . htmlspecialchars($av) .'"';
		}
		if($v || ($c && is_array($c) && sizeof($c)))
		{
			print ">${v}";
			if($c && is_array($c)) foreach($c as $p) FoafRDFRenderer::_dumpXML($p);
			print "</${n}>";
		}
		else
		{
			print "/>";
		}
		//print "\n";
	}

	public function sendHeaders($page)
	{
		if($this->bIsGood) 
		{
			if($this->contentType)
			{
				$page->setContentType($this->contentType);
			} else {
				$page->setContentType('application/rdf+xml');
			}
		}
		parent::sendHeaders($page);
	}

	public function startPage($page)
	{
		if(!$this->bIsGood) 
		{
			parent::startPage($page);
			return;
		}
?>
<rdf:RDF
      xmlns:admin="http://webns.net/mvcb/"
      xmlns:bio="http://purl.org/vocab/bio/0.1/"
      xmlns:dc="http://purl.org/dc/elements/1.1/"
      xmlns:geo="http://www.w3.org/2003/01/geo/wgs84_pos#"
      xmlns:foaf="http://xmlns.com/foaf/0.1/"
      xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
      xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
      xmlns:rel="http://www.purl.org/vocab/relationship/"
      xmlns:trust="http://www.perceive.net/schemas/2002.1119/trust/trust.rdf#"
	>
<foaf:PersonalProfileDocument rdf:about="<?php echo htmlspecialchars($page->myUrl());?>">
  <foaf:maker rdf:resource="#me"/>
  <foaf:primaryTopic rdf:resource="#me"/>
  <admin:generatorAgent rdf:resource="http://matt.wronka.org/stuff/projects/murder/"/>
  <admin:errorReportsTo rdf:resource="mailto:murder@projects.matt.wronka.org"/>
</foaf:PersonalProfileDocument>
<?php
	}

	public function endPage($page)
	{
		if(!$this->bIsGood) 
		{
			parent::endPage($page);
			return;
		}
		?></rdf:RDF><?php
	}
}

registerp(array('application/rdf+xml' => "FoafRDFRenderer"), "/");
registerp(array('application/rdf+xml' => "FoafRDFRenderer"), "foaf.rdf");

?>
