<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

require_once(dirname(__FILE__) . '/openid.php');

class LightOpenIDRenderer extends DefaultRenderer
{
	private $user;
	private $bIsGood;
	private $contentType;
	private $lightopenid = false;
	private $openidUrl = false;
	private $validated = false;
	private $shouldLogout = false;

	public function __construct($page, $arguments, $contentType)
	{
		parent::__construct($page, $arguments);
		$this->contentType = $contentType;
		$this->bIsGood = true; 
		$this->lightopenid = new LightOpenID($_SERVER['SERVER_NAME']);
		if($arguments && is_array($arguments) && sizeof($arguments))
		{
			if($arguments[0] == 'logout')
			{
				$this->shouldLogout = true;
			}
		}
	}

	public function processRequest($page)
	{
		parent::processRequest($page);
		if($this->shouldLogout)
		{
			MurderAuth::setUser(false);
			$page->msg("Logged-out.");
		}
		$this->openidUrl = safeGet($_POST, 'openid_url');
		if($this->openidUrl)
		{
			$this->lightopenid->identity = $this->openidUrl;
			$this->optional = array('namePerson/friendly', 'contact/email', 'namePerson');
		} else {
			if($this->lightopenid->validate())
			{
				$this->openidUrl = $this->lightopenid->identity;
				$this->validated = true;
			}
		}
	}

	public function mainContent($page)
	{
		if(!$this->bIsGood)
		{
			parent::mainContent($page);
			return;
		}

		if($this->validated)
		{
			$attrs = $this->lightopenid->getAttributes();
			$login = safeGet($attrs, 'namePerson/friendly');
			if(!$login) $login = safeGet($attrs, 'namePerson');
			if(!$login) $login = safeGet($attrs, 'contact/email');
			if(!$login) $login = $this->openidUrl;
			$user = new AuthenticatedUser($login);
			$user->set('email', safeGet($attrs, 'contact/email'));
			$user->set('name', safeGet($attrs, 'namePerson'));
			$user->set('openid', $this->openidUrl);
			MurderAuth::setUser($user);
			print "<p>Welcome " . $user->getNickname() . "!</p>";
		}
		if($this->openidUrl && !$this->validated)
		{
			$authUrl = htmlentities($this->lightopenid->authUrl(true));
			print "<p>You must complete authentication through <a href=\"${authUrl}\">${authUrl}</a>.</p>";
		}
		else if (!$this->openidUrl)
		{
			$user = MurderAuth::getUser();
			if($user)
			{
				print "<p>You are currently signed-in as: " . $user->getOpenID() . "</p>";
				print "<p><a href='/~/openid/logout'>Logout</a>.</p>";
			}
?>
<form action="/~/openid/" method="POST" class="clear">
	<label for="openid_url" class="fullw">OpenID</label><br />
	<input id="openid_url" type="url" name="openid_url" class="fullw" /><br />
	<input type="submit" class="fullw dblh" /><br />
</form>
<?php
		}
	}

	public function sendHeaders($page)
	{
		if($this->bIsGood) 
		{
			if($this->contentType)
			{
				$page->setContentType($this->contentType);
			} else {
				$page->setContentType('application/xhtml+xml');
			}
		}
		parent::sendHeaders($page);
	}
}

registerp(array('text/xml' => "LightOpenIDRenderer"), "openid");
registerp(array('application/xhtml+xml' => "LightOpenIDRenderer"), "openid");
registerp(array('*/*' => "LightOpenIDRenderer"), "openid");
registerm('/~/openid/', 'OpenID');

MurderAuth::addAuthUrl("openid", "OpenID");

?>
