<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

class RequestAddRenderer extends PageRenderer
{
	private $bIsGood = true;
	private $bMadeRequest = false;

	public function __construct($page, $arguments, $contentType)
	{
		parent::__construct($page, $arguments);
		if($arguments && is_array($arguments) && sizeof($arguments))
		{
			$this->bIsGood = false;
		}
		else $this->bIsGood = ($page->getUser() != false);
	}

	public function processRequest($page)
	{
		parent::processRequest($page);
		if($this->bIsGood)
		{
			$authuser = MurderAuth::getUser();
			if($authuser && safeGet($_POST, 'add'))
			{
				$this->bMadeRequest = true;
				#### Create an eMail with the user information!
				$attached = '';
				$target   = $page->getUser();
				$dir      = $target->getHome() . '/' . config("USER_BASE") ;
				$acontact = $authuser->getNickname();
				$name     = $target->getName();
				if(!$name) $name = $target->getNickname();
				$foaf     = safeGet($_POST, 'foaf');
				if($foaf) $attached .= "foaf\t${foaf}\r\n";
				foreach($authuser->getMap() as $k => $v)
				{
					$attached .= "${k}\t${v}\r\n";
				}
				## TODO: We could do something better if we didn't mind PEAR dependancies
				$subject = "Murder Request From " . $authuser->getNickname();
				$msg = "Dear ${name},\n"
				      ."A member of the Internet has asked that you acknowledge it.\r\n"
				      ."If you know this person (${acontact}) you may do so by\r\n"
				      ."adding the following lines to a new file in <${dir}/knows/>.\r\n"
				      ."(This file should be a simple nickname for the person.)\r\n\r\n"
				      ."=============== Begin File After This Line =================\r\n"
				      . $attached
				      ."===============  End File Before This Line =================";
				murder_mail($target, $subject, $msg);
			}
		} 
		else $page->setStatus(404);
	}

	public function mainContent($page)
	{
		if(!$this->bIsGood)
		{
			parent::mainContent($page);
			return;
		}
		$user = $page->getUser();
		$name = $user->getName(); if(!$name) $name = $user->getNickname();
		if(RequestAddRenderer::authuserIsKnown())
		{
			print "<p>You are already listed by ${name}.</p>";
			return;
		}
		$authuser = MurderAuth::getUser();
		##### Ok!  Let's handle our own rendering.
		if(!$authuser)
		{
			print "<h2>Request listing by ${name}</h2>";
			print "<p>You need to <a href='/~/openid/'>authenticate</a> first.</p>";
		}
		else if(!$this->bMadeRequest)
		{
			print "<h2>Request listing by ${name}</h2>";
?><p>If you complete this request, we will send <?php print $name; ?> a message on
	your behalf indicating that you would like to be added to the list of people
	<?php print $user->getLogin(); ?> knows.</p>
	<p>The following information will be included in the request:</p>
	<form action="request.add" method="POST">
		<?php 
			print "OpenID: " . $authuser->getOpenID() ."<br />";
			if($authuser->getName())
				print "Name: " . $authuser->getName() ."<br />";
			if($authuser->getNickname() && ($authuser->getNickname() != $authuser->getOpenID()))
				print "Nickname: " . $authuser->getNickname() ."<br />";
			if($authuser->getEMail())
				print "eMail: " . $authuser->getEMail() ."<br />";
		?>
		<label for="foaf_url">FOAF RDF File:</label>
		<input type="url" id="foaf_url" name="foaf" class="fullw" /> <br />
		<input type="submit" name="add" value="Send Request" class="fullw" />
	</form>
<?php		} else {
			print "<h2>Request sent to ${name}</h2>";
?><p>This may take some time, depending on how busy <?php print $user->getLogin(); ?> is.
	In the mean time, nothing would prohibit you from adding <?php print $name; ?>
	to your own FOAF file.  If you have your own 
	<a href="http://matt.wronka.org/stuff/projects/murder/" class="murdername">murder</a>
	installation, you can add <?php print $name; ?> to your own network.
</p><?php
		}
	}

	public static function authUserIsKnown()
	{
		$page = page();
		$pageUser = $page->getUser();
		if(!$pageUser) return false;
		$authuser = MurderAuth::getUser();
		if($authuser)
		{
			# are we already known?
			$knows = $pageUser->getKnownPeople();
			foreach($knows as $k)
			{
				if($k->getOpenID() == $authuser->getOpenID())
				{
					return true;
				}
			}
		}
		return false;
	}

	public static function renderMenu()
	{
		if(page()->getUser() && !RequestAddRenderer::authUserIsKnown())
		return "<a href='request.add'>Request Addition</a>";
	}
}

function request_add_renderer_render_menu() { return RequestAddRenderer::renderMenu(); }

registerp(array('text/xml' => "RequestAddRenderer"), "request.add");
registerp(array('application/xhtml+xml' => "RequestAddRenderer"), "request.add");
registerp(array('*/*' => "RequestAddRenderer"), "request.add");
PluginStore::getInstance()->addMenu('^/~.+/$', 'request_add_renderer_render_menu');


?>
