<?php
# Copyright (C) 2011 Matthew Wronka <murder@projects.matt.wronka.org>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, version 3 of the
# License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

class WebRenderer extends PageRenderer
{

	private $user;
	private $path;
	private $pathCorrected = false;
	private $fspath;
	private $bIsGood;
	private $error = 200;

	public function __construct($page, $arguments, $contentType)
	{
		parent::__construct($page, $arguments);
		$this->user = $page->getUser();
		if(!$this->user || !$this->user->hasWeb())
		{
			$this->bIsGood =  false;
			return;
		}
		if(!isset($arguments) || !$arguments)
		{
			$this->path = '/';
		}
		else
		{
			$cleanpath = array();
			foreach($arguments as $arg)
			{
				if($arg != '..')
				{
					$cleanpath[]=$arg;
				}
				else
				{
					if(null === array_pop($cleanpath))
					{
						// Do we care?
					}
				}
			}
			$this->path = '/' . implode('/', $cleanpath);
		}
		$this->fspath  =  $this->resolvePath($this->user->getHome() . '/' . config('WEB_DIR') . $this->path);
		$this->bIsGood =  is_readable($this->fspath);
	}

	protected function resolvePath($path)
	{
		if(is_readable($path) && !is_dir($path)) return $path;	// basically a no-op
		$dir = dirname($path);
		$base= basename($path);
		if(is_dir($path))
		{
			$index = $this->getIndex($path);
			if(!$index) 
			{
				$this->error = 403;
				return false;
			}
			if($dir != $path)
			{
				$this->pathCorrected = $path .'/';
				return $path .'/'. $index;
			}
			return $path . $index;
		}
		else
		{
			$fixed = $this->resolveExact(scandir($dir), $base);
			if(!$fixed) 
			{
				$this->error = 404;
				return false;
			}
			$this->pathCorrected = dirname($this->path) . '/' . $fixed;
			return $dir . '/' . $fixed;
		}
	}

	protected function resolveExact($files, $base)
	{
		foreach($files as $f)
		{
			$n = strrpos($f, '.');
			if($n > 0)
			{
				if($base == substr($f, 0, $n))
				{
					return $f;
				}
			}
		}
		return false;
	}

	protected function getIndex($path)
	{
		$indexes = explodea(' ', config('INDEX', 'WEB'));
		$contents = scandir($path);
		foreach($indexes as $i)
		{
			$ind = $this->resolveExact($contents, $i);
			if($ind) return $ind;
		}
		return false;
	}

	public function mainContent($page)
	{
		if($this->bIsGood)
		{
			readfile($this->fspath);
		} else {
			print "<h1>Unable to Serve</h1>\n";
			if($this->path)
			{
				print "<p>You asked for: " . htmlspecialchars($this->path) . "</p>";
			} else {
				print "<p>You asked for something non-sensical.</p>\n";
			}
		}
	}

	public function sendHeaders($page)
	{
		$page->setStatus($this->error);
		if($this->bIsGood) 
		{
			$stat = stat($this->fspath);
			if($stat)
			{
				if($modified = safeGet($stat, 'mtime'))
				{
					$page->setLastModified($modified);
				}
			}
			$mt = murder_mime_content_type($this->fspath);
			$page->setContentType($mt);
		}
		else
		{
			$page->setTitle($page->getTitle() . ' - '  . "There Was an Issue");
		}
		parent::sendHeaders($page);
	}

	public function startPage($page)
	{
		if(!$this->bIsGood) parent::startPage($page);
	}

	public function endPage($page)
	{
		if(!$this->bIsGood) parent::endPage($page);
	}
}

registerp(array("*/*" => "WebRenderer"), "web");

?>
